#!/usr/bin/perl -w
#
# This program is used to run strobe-classb.c.  It is called from
# relay-check.pl as well as being able to be run as a standalone
# program by editing config.pl for by making a config-$config.pl
# and running as 'strobe-classb.pl $config'.
#
# See the file COPYRIGHT for copyright information.
# (c) David Luyer, 1998.

use strict;
my $config = "config.pl";
$config = "config-$ARGV[0].pl"
  if(defined $ARGV[0]);
$config = "main process"
  if(defined $main::sub);
$| = 1
  if($config ne "main process");
my $kernel = `uname -r`;
die "kernel version too low - should be 2.0.35 release at least"
  if($kernel !~ /^(2\.(0\.(3[5-9]|[4-9][0-9])|[1-9])|[3-9])/);
undef $kernel;
require "$config"
  if($config ne "main process");
if($config::module eq "strobe-classb") {
  open(IMAX, "</proc/sys/kernel/inode-max");
  die "inode-max too low - I recommend 32768" if(int(<IMAX>) < 20000);
  close(IMAX);
  open(FMAX, "</proc/sys/kernel/file-max");
  die "file-max too low - I recommend 16384" if(int(<FMAX>) < 10000);
  close(FMAX);
}
die "$config must define \@config::ports"
  if(!defined @config::ports);
die "$config must define \@config::classbs"
  if(!defined @config::classbs);
die "$config must define \$config::dispose"
  if(!defined $config::dispose);
die "$config most define \$config::module"
  if(!defined $config::module);

sub make_strobe {
  my ($net, $port) = @_;
  my ($mac, $ip, @details);

  print STDERR "Building $config::module-$net-$port...";
  @details = `/sbin/ifconfig eth0`;
  for $_ (@details) {
    $mac = "-DMACADDR3=0x$1$2" if(/HWaddr ..:..:..:..:(..):(..)/);
    $ip = "-DSRCIP=(unsigned long)(($1<<24)|($2<<16)|($3<<8)|$4)"
      if(/inet addr:(\d+)\.(\d+)\.(\d+)\.(\d+)/);
  }
  system("/usr/bin/gcc","-c","-Wall","-ansi","-pedantic","-O2","-fno-builtin",
         "-fomit-frame-pointer","-fwritable-strings","-malign-loops=2",
         "-malign-jumps=2","-DCLASSB=$net"."L","-DDSTPORT=$port",
         $ip, $mac, "-DTIMEOUT=30", "-DSRCPORT=1",
         "$config::module.c","-o","$config::module-$net-$port.o");
  system("/usr/bin/ld","-sNx","-o","$config::module-$net-$port",
         "$config::module-$net-$port.o");
  unlink("$config::module-$net-$port.o");
  if( -x "$config::module-$net-$port" ) {
    print STDERR "done.\n";
  } else {
    print STDERR "failed.\n";
    die "compile failed";
  }
}

my $classb;
my $i = 0;
my %printed;
@main::hosts = ()
  if(defined $main::sub);
for $classb (@config::classbs) {
  die "Invalid class B network: $classb"
    if($classb !~ /^(\d+)\.(\d+)$/);
  my $net = $1 * 256 + $2;
  my $port;
  for $port (@config::ports) {
    my $oport = $port;
    %printed = ();
    $port = getservbyname($port, 'tcp')
      if($port !~ /^(\d+)$/);
    die "Invalid port: $oport"
      if($port !~ /^(\d+)$/);
    make_strobe($net, $port)
      if( ! -x "$config::module-$net-$port" );
    $i++;
    open(STROBE_CLASSB, "-|") || exec ("./$config::module-$net-$port", "$i");
    while(<STROBE_CLASSB>) {
      die "Misc error from $config::module"
        if($_ eq "XXXX\n");
      die "Bad $config::module output $_"
        if(!/^(..)(..)\n$/);
      goto SKIP
        if(defined $printed{$_});
      $printed{$_} = 1;
      my ($i1, $i2) = (hex $1, hex $2);
      print (($_ = $classb.".$i1.$i2:$oport")."\n");
      push @main::hosts, $_
        if (defined $main::sub);
SKIP:
    }
    close(STROBE_CLASSB);
    unlink("$config::module-$net-$port")
      if($config::dispose == 1);
  }
}
# Since this may be 'required' by something like check-relay.pl
1;
